package gssim.db.mysql;

import gssim.db.utils.PreparedStatementAdapter;

import java.sql.Connection;
import java.sql.SQLException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * The Class AbstractDAO.
 * 
 * @author Marcin Krystek, Jarek Szymczak
 */
public abstract class AbstractDAO {

	private Connection conn;
	private PreparedStatementAdapter insert;
	private PreparedStatementAdapter update;
	private PreparedStatementAdapter delete;
	private PreparedStatementAdapter find;
	private PreparedStatementAdapter select;
	private String relationName;

	protected Log log = LogFactory.getLog(AbstractDAO.class);

	/**
	 * Instantiates a new abstract dao.
	 * 
	 * @param conn
	 *            database connection object
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected AbstractDAO(Connection conn) throws SQLException {
		this.conn = conn;
		this.insert = createInsertStmt();
		this.update = createUpdateStmt();
		this.delete = createDeleteStmt();
		this.find = createFindStmt();
		this.select = createSelectStmt();
	}

	/**
	 * Closes all the statements and database connection.
	 * 
	 * @throws SQLException
	 *             the SQL exception
	 */
	public void close() throws SQLException {
		this.insert.close();
		this.update.close();
		this.delete.close();
		this.find.close();
		this.conn.close();
	}

	/**
	 * Commits database operations.
	 * 
	 * @throws SQLException
	 *             the SQL exception
	 */
	public void commit() throws SQLException {
		this.conn.commit();
		log.info("Database operations are commited");
	}

	/**
	 * Creates the delete statement.
	 * 
	 * @return the prepared statement adapter
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected abstract PreparedStatementAdapter createDeleteStmt()
			throws SQLException;

	/**
	 * Creates the find statement.
	 * 
	 * @return the prepared statement adapter
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected abstract PreparedStatementAdapter createFindStmt()
			throws SQLException;

	/**
	 * Creates the insert statement.
	 * 
	 * @return the prepared statement adapter
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected abstract PreparedStatementAdapter createInsertStmt()
			throws SQLException;

	/**
	 * Creates the select statement.
	 * 
	 * @return the prepared statement adapter
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected abstract PreparedStatementAdapter createSelectStmt()
			throws SQLException;

	/**
	 * Creates the update statement.
	 * 
	 * @return the prepared statement adapter
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected abstract PreparedStatementAdapter createUpdateStmt()
			throws SQLException;

	/**
	 * Gets the connection.
	 * 
	 * @return the connection
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected Connection getConnection() throws SQLException {
		if (this.conn == null)
			throw new SQLException("Connection not initialized.");

		if (this.conn.isClosed())
			throw new SQLException("Connection is closed.");

		return this.conn;
	}

	/**
	 * Gets the delete statement.
	 * 
	 * @return the delete statement
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected PreparedStatementAdapter getDeleteStmt() throws SQLException {
		if (this.delete == null)
			throw new SQLException("Delete statement is null.");
		return this.delete;
	}

	/**
	 * Gets the find statement.
	 * 
	 * @return the find statement
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected PreparedStatementAdapter getFindStmt() throws SQLException {
		if (this.find == null)
			throw new SQLException("Find statement is null.");
		return this.find;
	}

	/**
	 * Gets the insert statement.
	 * 
	 * @return the insert statement
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected PreparedStatementAdapter getInsertStmt() throws SQLException {
		if (this.insert == null)
			throw new SQLException("Insert statement is null.");
		return this.insert;
	}

	/**
	 * Gets the select statement.
	 * 
	 * @return the select statement
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected PreparedStatementAdapter getSelectStmt() throws SQLException {
		if (this.select == null)
			throw new SQLException("Select statement is null.");
		return this.select;
	}

	/**
	 * Gets the update statement.
	 * 
	 * @return the update statement
	 * @throws SQLException
	 *             the SQL exception
	 */
	protected PreparedStatementAdapter getUpdateStmt() throws SQLException {
		if (this.update == null)
			throw new SQLException("Update statement is null.");
		return this.update;
	}

	/**
	 * Logs success.
	 * 
	 * @param operation
	 *            succesful operation name
	 */
	public void logSuccess(String operation) {
		log.info("Operation " + operation + " in relation " + relationName
				+ " finished with no errors");
	}

	/**
	 * Logs success.
	 * 
	 * @param operation
	 *            succesful operation name
	 * @param rowsAffected
	 *            amount of rows affected by the operation
	 */
	public void logSuccess(String operation, int rowsAffected) {
		log.info("Operation " + operation + " in relation " + relationName
				+ " finished with no errors, rows affected: " + rowsAffected);
	}

	/**
	 * Logs error.
	 * 
	 * @param operation
	 *            operation, druring which an error occured
	 * @param errorRow
	 *            row number, on which an error occured
	 */
	public void logError(String operation, int errorRow) {
		log.error("Error during " + operation + " operation in relation "
				+ relationName + ", error occured on row: " + errorRow);
	}

	/**
	 * Logs error.
	 * 
	 * @param operation
	 *            operation, druring which an error occured
	 * @param rowsAffected
	 *            amount of rows affected by the operation
	 * @param objectAffected
	 *            the object affected by the operation
	 */
	public void logError(String operation, int rowsAffected,
			Object objectAffected) {
		log.error("Error during " + operation + " operation in relation "
				+ relationName + ", rows affected: " + rowsAffected
				+ ", object affected: " + System.getProperty("line.separator")
				+ objectAffected);
	}

	/**
	 * Gets the relation name.
	 * 
	 * @return the relation name
	 */
	public String getRelationName() {
		return relationName;
	}

	/**
	 * Sets the relation name.
	 * 
	 * @param relationName
	 *            the new relation name
	 */
	public void setRelationName(String relationName) {
		this.relationName = relationName;
	}

}
